#include <ginfo_plugin.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <dirent.h>
#include <dlfcn.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <libxml/parser.h>
#include <curl/curl.h>
#include <curl/types.h>
#include <curl/easy.h>
#include <pthread.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <errno.h>

#define MAN_DIR "/usr/share/man/man1"
#define MAN_PATH "/usr/bin/man"

GINFO_INIT;

GtkWidget *create_man(void *arg) {
  FILE *man;
  char tmp[1024];
  char tmp2[1024];
  int i,j;

  GtkWidget *tv;
  GtkTextIter iter;
  GtkTextBuffer *buffer;

  tv = gtk_text_view_new ();
  gtk_widget_show (tv);
  gtk_container_set_border_width (GTK_CONTAINER (tv), 5);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (tv), FALSE);
  gtk_text_view_set_accepts_tab (GTK_TEXT_VIEW (tv), FALSE);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (tv), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (tv), GTK_WRAP_WORD);

  buffer=gtk_text_view_get_buffer (GTK_TEXT_VIEW (tv));
  gtk_text_buffer_get_start_iter(buffer, &iter);

  snprintf(tmp, sizeof(tmp), "%s -Tascii %s", MAN_PATH, (char *)arg);
  man=popen(tmp,"r");
  if(!man) ginfo_error("Man error...");

  if(!man) return NULL;
  while(!feof(man)) {
    i=fread(tmp, 1, 1024, man);
    if(i<1) continue;

    tmp[i]=0;

    for(i=j=0;i<strlen(tmp);i++) {
      if(tmp[i]!=0x8)
        tmp2[j++]=tmp[i];
      else
	j--;
    }
    tmp2[j]=0;

    gtk_text_buffer_insert (buffer, &iter, tmp2, -1);

  }

  pclose(man);

  return tv;
}

int
module_default (void)
{
  return 0;
}

char *
module_name (void)
{
  return "man2ginfo";
}

int
module_genre (void)
{
  return GINFO_GENRE_SYSTEM;
}

char **
module_words (void)
{
  char **list;
  DIR *d;
  struct dirent *dir;
  int n=0;

  if(!(d=opendir(MAN_DIR)))
    ginfo_error("Where is man1 ?");

  while((dir=readdir(d)) != NULL) {
    if(strcmp(dir->d_name + (strlen(dir->d_name) -3), ".gz")) continue;
    n++;
  }

  if (!(list = (char **) malloc (sizeof (char *) * (n+2))))
    ginfo_error ("Error: memory.");

  closedir(d);

  if(!(d=opendir(MAN_DIR)))
    ginfo_error("Where is man1 ?");

  n=0;
  while((dir=readdir(d)) != NULL) {
    if(strcmp(dir->d_name + (strlen(dir->d_name) -3), ".gz")) continue;

    list[n] = strdup(dir->d_name);
    n++;
  }

  closedir(d);

  list[n]=NULL;

  return list;
}

ginfo_docs *
module_docs (int *n)
{
  void *t;
  DIR *d;
  struct dirent *dir;
  char *buf;
  int i;

  if(!(d=opendir(MAN_DIR)))
    ginfo_error("Where is man1 ?");

  t = ginfo_easy_docs_init ();

  while((dir=readdir(d)) != NULL) {
    if(strcmp(dir->d_name + (strlen(dir->d_name) -3), ".gz")) continue;

    buf=strdup(dir->d_name);
    for(i=0;i<strlen(buf);i++) if(buf[i]=='.') {
	    buf[i]=0;
	    break;
    }

    ginfo_easy_docs_add (t, strdup(buf), create_man, strdup(buf), NULL);
    free(buf);
  }

  closedir(d);

  return ginfo_easy_docs_close (t, n);
}

