/* gInfo - Copyright (C) 2004 bakunin - Andrea Marchesini 
 *                                      <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "ginfo.h"

/* This function are for C modules. Read the doc inside of ginfo about */

/* GINFO_EASY_DOCS_*  */
typedef struct _ginfo_easy_docs_e_
{

  char *text;
  GtkWidget *(*func) (void *);
  void *arg;
  char *id;

  struct _ginfo_easy_docs_e_ *next;

} ginfo_easy_docs_e;

typedef struct
{

  int n;

  ginfo_easy_docs_e *e;
  ginfo_easy_docs_e *e_last;

} ginfo_easy_docs;

void *
ginfo_easy_docs_init (void)
{
  ginfo_easy_docs *t;

  if (!(t = (ginfo_easy_docs *) malloc (sizeof (ginfo_easy_docs))))
    ginfo_error ("Error: memory.");

  t->n = 0;
  t->e = NULL;
  t->e_last = NULL;

  return (void *) t;
}

int
ginfo_easy_docs_add (void *te, char *text, GtkWidget * (*func) (void *),
		     void *arg, char *id)
{
  ginfo_easy_docs *e = (ginfo_easy_docs *) te;
  ginfo_easy_docs_e *el;

  if (!e)
    return 1;


  if (!(el = (ginfo_easy_docs_e *) malloc (sizeof (ginfo_easy_docs_e))))
    ginfo_error ("Error: memory");

  el->text = text;
  el->func = func;
  el->arg = arg;
  el->id = id;

  el->next = NULL;

  if (!e->e)
    {
      e->e = e->e_last = el;
    }
  else
    {
      e->e_last->next = el;
      e->e_last = el;
    }

  e->n++;

  return 0;
}

ginfo_docs *
ginfo_easy_docs_close (void *te, int *k)
{

  ginfo_easy_docs *e = (ginfo_easy_docs *) te;
  ginfo_docs *list;
  ginfo_easy_docs_e *q;
  int n = 0;

  if (!e)
    return NULL;

  if (!(list = (ginfo_docs *) malloc (sizeof (ginfo_docs) * e->n)))
    ginfo_error ("Error: memory.");

  *k = e->n;

  while (e->e)
    {
      list[n].text = e->e->text;
      list[n].func = e->e->func;
      list[n].arg = e->e->arg;
      list[n].id = e->e->id;

      q = e->e;
      e->e = e->e->next;

      free (q);

      n++;
    }

  free (e);

  return list;
}

void
ginfo_easy_docs_list_free (void *t)
{
  free (t);
}

void
ginfo_easy_docs_free (void *te)
{
  ginfo_easy_docs *e = (ginfo_easy_docs *) te;
  ginfo_easy_docs_e *q;

  if (!e)
    return;

  while (e->e)
    {
      q = e->e;
      e->e = e->e->next;

      free (q);
    }

  free (e);
}

/* GINFO_EASY_TIPS_*  */
typedef struct _ginfo_easy_tips_e_
{

  char *text;
  GtkWidget *(*func) (void *);
  void *arg;
  char *id;

  struct _ginfo_easy_tips_e_ *next;

} ginfo_easy_tips_e;

typedef struct
{

  int n;

  ginfo_easy_tips_e *e;
  ginfo_easy_tips_e *e_last;

} ginfo_easy_tips;

void *
ginfo_easy_tips_init (void)
{
  ginfo_easy_tips *t;

  if (!(t = (ginfo_easy_tips *) malloc (sizeof (ginfo_easy_tips))))
    ginfo_error ("Error: memory.");

  t->n = 0;
  t->e = NULL;
  t->e_last = NULL;

  return (void *) t;
}

int
ginfo_easy_tips_add (void *te, char *text, GtkWidget * (*func) (void *),
		     void *arg, char *id)
{
  ginfo_easy_tips *e = (ginfo_easy_tips *) te;
  ginfo_easy_tips_e *el;

  if (!e)
    return 1;


  if (!(el = (ginfo_easy_tips_e *) malloc (sizeof (ginfo_easy_tips_e))))
    ginfo_error ("Error: memory");

  el->text = text;
  el->func = func;
  el->arg = arg;
  el->id = id;

  el->next = NULL;

  if (!e->e)
    {
      e->e = e->e_last = el;
    }
  else
    {
      e->e_last->next = el;
      e->e_last = el;
    }

  e->n++;

  return 0;
}

ginfo_tips *
ginfo_easy_tips_close (void *te, int *k)
{

  ginfo_easy_tips *e = (ginfo_easy_tips *) te;
  ginfo_tips *list;
  ginfo_easy_tips_e *q;
  int n = 0;

  if (!e)
    return NULL;

  if (!(list = (ginfo_tips *) malloc (sizeof (ginfo_tips) * e->n)))
    ginfo_error ("Error: memory.");

  *k = e->n;

  while (e->e)
    {
      list[n].text = e->e->text;
      list[n].func = e->e->func;
      list[n].arg = e->e->arg;
      list[n].id = e->e->id;

      q = e->e;
      e->e = e->e->next;

      free (q);

      n++;
    }

  free (e);

  return list;
}

void
ginfo_easy_tips_list_free (void *t)
{
  free (t);
}

void
ginfo_easy_tips_free (void *te)
{
  ginfo_easy_tips *e = (ginfo_easy_tips *) te;
  ginfo_easy_tips_e *q;

  if (!e)
    return;

  while (e->e)
    {
      q = e->e;
      e->e = e->e->next;

      free (q);
    }

  free (e);
}

void
ginfo_error (char *a)
{
  fprintf (stderr, "* %s\n", a);
  exit (1);
}

int
ginfo_check_genre (char *g)
{

  if (!strcasecmp (g, "SYSTEM"))
    return GINFO_GENRE_SYSTEM;
  if (!strcasecmp (g, "NETWORK"))
    return GINFO_GENRE_NETWORK;
  if (!strcasecmp (g, "AUDIO"))
    return GINFO_GENRE_AUDIO;
  if (!strcasecmp (g, "VIDEO"))
    return GINFO_GENRE_VIDEO;
  if (!strcasecmp (g, "STREAM"))
    return GINFO_GENRE_STREAM;
  if (!strcasecmp (g, "OFFICE"))
    return GINFO_GENRE_OFFICE;
  if (!strcasecmp (g, "CODING"))
    return GINFO_GENRE_CODING;

  return 0;
}

char **
split_string (char *str)
{
  char **t, *k, **w;
  int n = 1;

  k = str;

  while (str)
    {
      n++;

      while (str && *str && *str != ' ' && *str != '\t')
	str++;

      if (!*str || !str)
	break;

      str++;
    }

  if (!(t = (char **) malloc (sizeof (char *) * n)))
    fatal ("Error: memory.");
  w = t;

  str = k;

  while (str)
    {
      *t = str;
      t++;

      while (str && *str && *str != ' ' && *str != '\t')
	str++;

      if (!*str || !str)
	break;

      *str = 0;
      str++;
    }
  *t = NULL;

  return w;
}

char **
ginfo_check_words (char *w)
{
  if (!w || !*w)
    return NULL;

  return split_string (w);
}

/* EOF */
